@@ -1,6 +1,9 @@
 package Echo;
-sub Echo::say {
+use Test::More;
+sub say {
     my $cb = pop;
+    is(ref($cb), 'CODE', "The last arg is a code ref");
+
     my ($class, @args) = @_;
     $cb->($class, @args);
 }
